using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.XPath;
using System.Xml;

namespace InfiniTec.Exchange
{
    /// <summary>
    /// Represents a multi-valued property of an Exchange <see cref="Item"/>
    /// </summary>
    /// <typeparam name="T">The type of the property</typeparam>
    public class MultiValuedProperty<T>: Property
    {
        PropertyValueCollection<T> _Values;

        /// <summary>
        /// Returns a list containing the elements of the property.
        /// </summary>
        public IList<T> Values
        {
            get { return _Values; }
        }

        internal sealed override void Load(XPathNavigator navigator)
        {
            navigator.MoveToChild(XPathNodeType.Element);

            do
            {
                _Values.Add(DecodeValue<T>(navigator.InnerXml));
            } while (navigator.MoveToNext(XPathNodeType.Element));
        }

        internal sealed override void SaveTo(XmlWriter writer)
        {
            writer.WriteStartElement(Name, Namespace);

            writer.WriteAttributeString("dt", Namespaces.DataType, "mv." + GetDataType<T>());
            foreach (T value in Values)
            {
                writer.WriteElementString("v", Namespaces.Xml, EncodeValue<T>(value));
            }

            writer.WriteEndElement();
        }

        /// <summary>
        /// Creates a new instance of this class using the specified namespace and local
        /// name.
        /// </summary>
        /// <param name="ns">The namespace of the property</param>
        /// <param name="localName">The local name of the property</param>
        /// <param name="values">The initial values of the property</param>
        public MultiValuedProperty(string ns, string localName, params T[] values): base(ns, localName)
        {
            _Values = new PropertyValueCollection<T>(this);
            _Values.AddRange(values);

            SetChanged();
        }
        /// <summary>
        /// Creates a new instance of this class using the specified namespace and local
        /// name.
        /// </summary>
        /// <param name="fullQualifiedName">The full qualified name of the property</param>
        /// <param name="values">The initial values of the property</param>
        public MultiValuedProperty(string fullQualifiedName,
            params T[] values): base(fullQualifiedName)
        {
            _Values = new PropertyValueCollection<T>(this);
            _Values.AddRange(values);

            SetChanged();
        }


    }
}
