using System;
using System.Collections.Generic;
using System.Text;
using InfiniTec.Threading;
using System.Threading;
using InfiniTec.Exchange.WebDav;

namespace InfiniTec.Exchange
{
    /// <summary>
    /// This class provided base functionality and properties for all WebDav related helper objects.
    /// </summary>
    public abstract class ObjectBase
    {
        private int _AsyncOperationPending;
        private AsyncOperation _PendingOperation;
        private Connection _Connection;
        private Methods _Methods;
        private Uri _Url;

        /// <summary>
        /// Returns the <see cref="Uri"/> of the current instance.
        /// </summary>
        public Uri Url
        {
            get { return _Url; }
        }

        /// <summary>
        /// Returns an instance of a <see cref="Methods"/> class which can be used for WebDav requests.
        /// </summary>
        protected Methods Methods
        {
            get { return _Methods; }
        }

        /// <summary>
        /// Returns the connection associated with this instance.
        /// </summary>
        public Connection Connection
        {
            get { return _Connection; }
        }

        /// <summary>
        /// Initiates a new instance of this class using the specified settings.
        /// </summary>
        /// <param name="path">The path of the item on the Exchange folder.</param>
        /// <param name="connection">The connection to use</param>
        protected ObjectBase (string path, Connection connection): this(
            connection.CreateUrl(path), connection)
        {
            
        }
        /// <summary>
        /// Initiates a new instance of this class using the specified settings.
        /// </summary>
        /// <param name="url">The url of the item on the Exchange folder.</param>
        /// <param name="connection">The connection to use</param>
        protected ObjectBase(Uri url, Connection connection)
        {
            if (!connection.IsValidUrl(url)) throw new InvalidConnectionException("The specified connection is not suitable for the given url");

            _Connection = connection;
            _Url = url;
            _Methods = new Methods(connection);
        }

        /// <summary>
        /// Checks whether an asynchronous operation is already in progress and throws an exception in that case.
        /// After the call to this method, this instance is marked as "async operation is pending".
        /// </summary>
        protected void InitiatePendingAsyncOperation()
        {
            if (Interlocked.Exchange(ref _AsyncOperationPending, 1) == 1)
                throw new InvalidOperationException("Another async operation is currently in progress.");
        }

        /// <summary>
        /// Registers the specified operation as pending async operation to prevent it from being garbage collected.
        /// </summary>
        /// <param name="operation">The operation to register</param>
        protected void RegisterPendingAsyncOperation(AsyncOperation operation)
        {
            _PendingOperation = operation;
        }

        /// <summary>
        /// Unregisters the pending operation and remove the "Async operation is pending" flag.
        /// </summary>
        protected void UnregisterPendingAsyncOperation()
        {
            _PendingOperation = null;
            Interlocked.Exchange(ref _AsyncOperationPending, 0);
        }

        /// <summary>
        /// Aborts a pending async operation.
        /// </summary>
        public void AbortPendingAsyncOperation()
        {
            if (_PendingOperation == null) throw new InvalidOperationException("No async operation is currently in progress");

            _PendingOperation.Abort();
        }
    }
}
