using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.XPath;
using System.Xml;
using System.Net;

namespace InfiniTec.Exchange
{
    /// <summary>
    /// This class contains the base functionality for a property of an Exchange item.
    /// </summary>
    public class Property
    {
        private string _Namespace;
        private bool _HasChanged;
        private string _Name;
        private HttpStatus _Status;

        /// <summary>
        /// If the property was created during a refresh operation, 
        /// this value returns the Http status of that request.
        /// </summary>
        public HttpStatus Status
        {
            get { return _Status; }
        }

        /// <summary>
        /// Returns the full qualifed name of the property.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return FullQualifiedName;
        }
        /// <summary>
        /// Returns true, if the value of the property has changed.
        /// False, otherwise.
        /// </summary>
        public bool HasChanged
        {
            get { return _HasChanged; }
        }

        /// <summary>
        /// Returns the namespace of the item.
        /// </summary>
        public string Namespace
        {
            get { return _Namespace; }
        }

        /// <summary>
        /// Returns the local name of the property.
        /// </summary>
        public string Name
        {
            get { return _Name; }
        }

        /// <summary>
        /// Returns the full qualified name of the property.
        /// </summary>
        public string FullQualifiedName
        {
            get { return _Namespace + _Name; }
        }

        private Property()
        {
            _Status = new HttpStatus("HTTP/1.1 200 OK");
        }

        /// <summary>
        /// Creates a new instance of this class using the specified name.
        /// </summary>
        /// <param name="ns">The namespace of the proerty</param>
        /// <param name="localName">The local name of the property</param>
        protected Property(string ns, string localName): this()
        {
            _Namespace = ns;
            _Name = localName;
        }
        /// <summary>
        /// Creates a new instance of this class using the specified name.
        /// </summary>
        /// <param name="fullQualifiedName">The full qualified name of the property</param>
        protected Property(string fullQualifiedName):this()
        {
            ParsePropertyName(fullQualifiedName, out _Namespace, out _Name);
        }

        /// <summary>
        /// Parses a full qualified property and returns the namespace and the local name.
        /// </summary>
        /// <param name="fullQualifiedName">The full qualified name of the property</param>
        /// <param name="ns">The namespace of the property</param>
        /// <param name="localName">The local name of the property</param>
        public static void ParsePropertyName(string fullQualifiedName, out string ns, out string localName)
        {
            int index;
            // Custom Outlook properties don't have any namespaces, 
            // so LastIndexOf maybe -1
            index = Math.Max(fullQualifiedName.LastIndexOfAny(new char[] { '/', ':', '#' }) + 1, 0);

            ns = fullQualifiedName.Substring(0, index);
            localName = fullQualifiedName.Substring(index);
        }

        /// <summary>
        /// Loads the value of the property from the navigator.
        /// </summary>
        /// <param name="navigator">A <see cref="XPathNavigator"/> from which
        /// the value is read.</param>
        internal virtual void Load(XPathNavigator navigator) { }

        /// <summary>
        /// Creates <see cref="Property"/> based on the provided 
        /// xml data.
        /// </summary>
        /// <param name="navigator">An <see cref="XPathNavigator"/>
        /// containing the data of the property.</param>
        /// <param name="status">The <see cref="HttpStatus"/> of the refresh-status</param>
        /// <returns>A <see cref="Property"/> instance.</returns>
        public static Property Create(XPathNavigator navigator, HttpStatus status)
        {
            Property p;
            string dataType;
            bool isMultiValued;

            dataType = GetDataType(navigator, out isMultiValued);

            if (status.StatusCode != HttpStatusCode.OK)
            {
                p = new Property(navigator.NamespaceURI, navigator.LocalName);
            }
            else
            {
                p = CreateProperty(navigator.NamespaceURI, navigator.LocalName,
                    dataType, isMultiValued);
            }
            p._Status = status;
            p.Load(navigator);

            p._HasChanged = false;

            return p;
        }

        private static Property CreateProperty(
            string ns, string name,
            string dataType, bool isMultiValued)
        {
            switch (dataType)
            {
                case "boolean": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<bool>(ns, name) : 
                    (Property) new Property<bool>(ns, name);
                case "dateTime.tz": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<DateTime>(ns, name) : 
                    (Property) new Property<DateTime>(ns, name);
                case "i2": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<Int16>(ns, name) : 
                    (Property) new Property<Int16>(ns, name);
                case "int": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<int>(ns, name) : 
                    (Property) new Property<int>(ns, name);
                case "i8": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<long>(ns, name) : 
                    (Property) new Property<long>(ns, name);
                case "r4": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<Single>(ns, name) : 
                    (Property) new Property<Single>(ns, name);
                case "fixed.14.4": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<double>(ns, name) : 
                    (Property) new Property<double>(ns, name);
                case "float": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<float>(ns, name) : 
                    (Property) new Property<float>(ns, name);
                case "uuid": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<Guid>(ns, name) : 
                    (Property) new Property<Guid>(ns, name);
                case "bin.base64": return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<byte[]>(ns, name) : 
                    (Property) new Property<byte[]>(ns, name);
                default: return (isMultiValued) ? 
                    (Property) new MultiValuedProperty<string>(ns, name) : 
                    (Property) new Property<string>(ns, name);
            }
        }

        private static string GetDataType(XPathNavigator navigator, out bool isMultiValued)
        {
            string dataType;

            dataType = navigator.GetAttribute("dt", Namespaces.DataType);

            isMultiValued = false;
            if (string.IsNullOrEmpty(dataType)) return "string";

            if (dataType.StartsWith("mv.", StringComparison.CurrentCultureIgnoreCase))
            {
                isMultiValued = true;
                dataType = dataType.Substring(3);
            }
            return dataType;
        }

        internal static T DecodeValue<T>(string value)
        {
            if (typeof(T) == typeof(bool)) return (T) (object) XmlConvert.ToBoolean(value);
            else if (typeof(T) == typeof(DateTime)) return (T) (object) XmlConvert.ToDateTime(value, XmlDateTimeSerializationMode.Local);
            else if (typeof(T) == typeof(Int16)) return (T) (object) XmlConvert.ToInt16(value);
            else if (typeof(T) == typeof(int)) return (T) (object) XmlConvert.ToInt32(value);
            else if (typeof(T) == typeof(long)) return (T) (object) XmlConvert.ToInt64(value);
            else if (typeof(T) == typeof(double)) return (T) (object) XmlConvert.ToDouble(value);
            else if (typeof(T) == typeof(float)) return (T) (object) XmlConvert.ToSingle(value);
            else if (typeof(T) == typeof(Guid)) return (T) (object) XmlConvert.ToGuid(value);
            else if (typeof(T) == typeof(byte[])) return (T) (object) Convert.FromBase64String(value);

            return (T) (object) value;
        }

       
        internal string EncodeValue<T>(T value)
        {
            if (typeof(T) == typeof(bool)) return (bool) (object) value ? "1" : "0";
            else if (typeof(T) == typeof(byte[])) return Convert.ToBase64String((byte[]) (object) value);
            else if (typeof(T) == typeof(DateTime)) return ((DateTime) (object) value).ToUniversalTime().ToString("yyyy-MM-ddTHH:mm:sszzz");
            else return value.ToString();
        }

        internal string GetDataType<T>()
        {
            if (typeof(T) == typeof(bool)) return "boolean";
            else if (typeof(T) == typeof(byte[])) return "bin.base64";
            else if (typeof(T) == typeof(DateTime)) return "dateTime.tz";
            else if (typeof(T) == typeof(int)) return "int";
            else if (typeof(T) == typeof(short)) return "i2";
            else if (typeof(T) == typeof(long)) return "i8";
            else if (typeof(T) == typeof(float)) return "float";
            else if (typeof(T) == typeof(double)) return "fixed.14.4";
            else if (typeof(T) == typeof(Guid)) return "uuid";
            else return "string";
        }

        internal void SetChanged()
        {
            _HasChanged = true;
        }

        internal virtual void SaveTo(XmlWriter writer) { }

    }
}