using System;
using System.Collections.Generic;
using System.Text;
using InfiniTec.Threading;
using InfiniTec.Exchange.WebDav;

namespace InfiniTec.Exchange
{
    /// <summary>
    /// This class performs a search of an Exchange public or private
    /// folder.
    /// </summary>
    public class Searcher: ObjectBase
    {
        /// <summary>
        /// This event is invoked when the progress of the search operation
        /// changed.
        /// </summary>
        public event EventHandler<SearchProgressChangedEventArgs> ProgressChanged;
        /// <summary>
        /// This event is invoked when the the search operation has completed.
        /// </summary>
        public event EventHandler<SearchCompletedEventArgs> Completed;

        private int _PageSize;
        private SearchScope _SearchScope;
        private IList<string> _PropertiesToLoad;
        private string _SearchOrder;
        private string _Constraints;
        private SearchScope _Scope;

        /// <summary>
        /// Gets or sets the <see cref="SearchScope"/> of the search operation.
        /// </summary>
        public SearchScope Scope
        {
            get { return _Scope; }
            set { _Scope = value; }
        }

        /// <summary>
        /// Gets or sets the constraints of the operation.
        /// </summary>
        /// <remarks>
        /// The constraints is something like
        /// <code>"DAV:ishidden" = false</code>
        /// </remarks>
        public string Constraints
        {
            get { return _Constraints; }
            set { _Constraints = value; }
        }

        /// <summary>
        /// Gets or sets the sorting key of the search.
        /// </summary>
        /// <remarks>The search order can be something like
        /// <code>"DAV:displayname" ASC</code>
        /// </remarks>
        public string SearchOrder
        {
            get { return _SearchOrder; }
            set { _SearchOrder = value; }
        }

        /// <summary>
        /// Provides access to a collection containing the properties to search for.
        /// If this collection is empty, all properties are returned.
        /// </summary>
        public IList<string> PropertiesToLoad
        {
            get { return _PropertiesToLoad; }
        }

        /// <summary>
        /// Gets or sets the <see cref="SearchScope"/> of the search operation.
        /// </summary>
        public SearchScope SearchScope
        {
            get { return _SearchScope; }
            set { _SearchScope = value; }
        }
        /// <summary>
        /// Gets or sets the number of items returned by each iteration.
        /// </summary>
        public int PageSize
        {
            get { return _PageSize; }
            set { _PageSize = value; }
        }

        /// <summary>
        /// Calls the <see cref="M:ProgressChanged"/> event.
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnProgressChanged(SearchProgressChangedEventArgs e)
        {
            if (ProgressChanged != null) ProgressChanged(this, e);
        }

        /// <summary>
        /// Calls the <see cref="M:ProgressCompleted"/> event.
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnCompleted(SearchCompletedEventArgs e)
        {
            if (Completed != null) Completed(this, e);
        }

        /// <summary>
        /// Synchronously starts the search operation.
        /// </summary>
        /// <returns></returns>
        public IList<Item> Find()
        {
            using (AsyncOperation<SearchProgress, IList<Item>> op = GetSearchOperation())
            {
                op.Run();
                op.WaitForCompletion();

                return op.Result;
            }
        }

        /// <summary>
        /// Asynchronously starts the search operation.
        /// </summary>
        public void FindAsync()
        {
            AsyncOperation<SearchProgress, IList<Item>> operation;

            InitiatePendingAsyncOperation();

            operation = GetSearchOperation();

            RegisterPendingAsyncOperation(operation);

            operation.Completed += SearchAsyncCompleted;
            operation.Error += SearchAsyncError;
            operation.ProgressChanged += SearchProgressChanged;

            operation.Run();
        }

        void SearchProgressChanged(object sender, ProgressChangedEventArgs<SearchProgress> e)
        {
            SearchProgressChangedEventArgs args;

            args = new SearchProgressChangedEventArgs(e.Status);
            OnProgressChanged(args);

            e.Cancel = args.Cancel;
        }

        void SearchAsyncError(object sender, ExceptionEventArgs e)
        {
            UnregisterPendingAsyncOperation();
            OnCompleted(new SearchCompletedEventArgs(e.Exception));
        }

        void SearchAsyncCompleted(object sender, CompletedEventArgs<IList<Item>> e)
        {
            UnregisterPendingAsyncOperation();
            if (e.Status == OperationStatus.Completed)
                OnCompleted(new SearchCompletedEventArgs(e.Result));
            else
                OnCompleted(new SearchCompletedEventArgs(true));
        }

        /// <summary>
        /// Returns the search operation used to perform the search based
        /// ojn the specified settings.
        /// </summary>
        /// <returns></returns>
        public virtual AsyncOperation<SearchProgress, IList<Item>> GetSearchOperation()
        {
            return new AsyncOperation<SearchProgress, IList<Item>>(SearchOperationInternal());
        }

        private IEnumerable<OperationAction> SearchOperationInternal()
        {
            SearchSettings settings;
            int itemsFound;
            int page;
            SearchProgress progress;
            List<Item> items;
            ProgressUpdate<SearchProgress> update;

            progress = new SearchProgress();
            update = new ProgressUpdate<SearchProgress>(progress);
            items = new List<Item>();

            settings = new SearchSettings(_PropertiesToLoad);
            settings.Constraints = Constraints;
            settings.OrderBy = SearchOrder;
            settings.Scope = Scope;
            settings.Ranges.Add(new Range(0, PageSize-1));
            page = 0;

            do
            {
                using (AsyncOperation<RequestStatus, IList<Item>> op = Methods.Search(Url, settings))
                {
                    yield return new NestedOperation<SearchProgress>(op, true, update);

                    itemsFound = op.Result.Count;

                    items.AddRange(op.Result);
                    progress.RequestStatus = RequestStatus.None;
                    if (itemsFound > 0)
                    {
                        progress.Range = new Range(page * PageSize, page * PageSize + itemsFound - 1);
                    }
                    else
                    {
                        progress.Range = null;
                    }
                    progress.Items = op.Result;

                    yield return update;
                }

                page++;

                settings.Ranges.Clear();
                settings.Ranges.Add(new Range(page * PageSize, (page + 1) * PageSize-1));
            } while (itemsFound == PageSize);

            yield return new OperationResult<IList<Item>>(items.AsReadOnly());
        }

        /// <summary>
        /// Creates a new instance of this class using the specified settings and
        /// a PageSize of 200.
        /// </summary>
        /// <param name="searchRoot">The root of the search operation.</param>
        /// <param name="connection">The <see cref="Connection"/> to use.</param>
        /// <param name="propertiesToLoad">The properties to load. This parameter is optional.</param>
        public Searcher(string searchRoot, Connection connection, params string[] propertiesToLoad): 
            base(searchRoot, connection)
        {
            Initialize(propertiesToLoad);
        }

       
        internal Searcher(Uri searchRoot, Connection connection, string[] propertiesToLoad):
            base(searchRoot, connection)
        {
            Initialize(propertiesToLoad);
        }
 
        private void Initialize(string[] propertiesToLoad)
        {
            _PageSize = 200;
            _PropertiesToLoad = new List<string>(propertiesToLoad);
        }
    }
}
